@@ -456,7 +456,7 @@ class BaseFlow(JAMLCompatible, ExitStack, metaclass=FlowType):
         # set env only before the pod get started
         if self._env:
             for k, v in self._env.items():
-                os.environ[k] = v
+                os.environ[k] = str(v)
 
         for v in self._pod_nodes.values():
             self.enter_context(v)
